`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Company:         M. A. Morris & Associates 
// Engineer:        Michael A. Morris 
// 
// Create Date:     06:03:27 06/19/2013 
// Design Name:     Control CPLD for 1004-0001 M65C02 Test CCA 
// Module Name:     1004CPLD 
// Project Name:    C:\XProjects\ISE10.1i\1004CPLD
// Target Devices:  Xilinx Coolrunner 3 CPLDs: XC2C32A-6VQ44, XC2C64A-7VQ44 
// Tool versions:   Xilinx ISE 10.1i SP3
// 
// Description:
//
//  This module provides the control of the FPGA configuration mode, configura-
//  tion serial PROM version select, and the nPUDC control pins. Further, the
//  CPLD is also connected to various interrupt pins on the FPGA so that those
//  features of the M65C02 core can be tested in a controlled manner. The CPLD
//  expects that the M65C02 processor will utilize an SPI interface, the confi-
//  guration PROM interface, to access internal registers.
//
//  The implementation of the SPI slave interface may require the use of the 
//  larger of the two devices supported on the 1004-0001 M65C02 Test CCA.
//
//  The initial version of the module will only require the use of the XC2C32A
//  CPLD. Its purpose will be to dynamically drive the M[2:0], VS[2:0], and the
//  nPUDC/Spare pins of the FPGA. On power on, a 2.7V MCP130 voltage supervisor
//  asserts nPROG to the FPGA and the CPLD. This signal should enable the output
//  drivers for the M[2:0], VS[2:0], and nPUDC/Spare pins. Following the release
//  of nPROG, the FPGA will sample these pins to determine the configuration
//  mode, Serial EPROM type and SCK speed, and whether PUs should be enabled 
//  during configuration. During this interval, the CPLD should be actively 
//  driving the pins to the programmed levels. Internal PUs in the FPGA on these
//  pins means that the CPLD need only sink current when one of these pins is 
//  set to output a zero. If any of these 7 pins is set to drive a logic 1
//  during configuration, then it should do so through a tri-stated output
//  driver. In other words, the CPLD should simulate an open-drain driver.
//  
//  The FPGA's DONE pin is an input to the CPLD. It will be asserted high when
//  configuration is complete and the FPGA logic is in active. When DONE is a 
//  logic 1, these 7 configuration mode pins should be tri-stated. Series resis-
//  tors in the signal trace between the CPLD and the FPGA will be sufficient
//  to avoid any contention between the CPLD and the FPGA. All 7 of these pins
//  are used by the M65C02 processor implementation. On the M65C02 Test CCA, the
//  pins are connected to various external devices, i.e. RAMs and ROMs.
// 
//  Immediately after the M65C02's processor core is released from reset, these
//  7 pins are not actively used. This is because the core is configured to use
//  an internal block RAM for booting the M65C02 system. Thus the XA[3:1] and 
//  A[15:13] signal lines to which M[2:0] and VS[2:0] connect, respectively, are
//  not actively involved in an memory read operation. The nPUDC/Spare signal is
//  configured on the M65C02 as an input with a PU; it will not be meaningfully
//  affected if the CPLD continues to actively drive the pin low after DONE goes
//  high. 
//
// Dependencies: 
//
// Revision:
//
//  0.01    13F19   MAM     Initial creation - targeting XC2C32A-6VQ44 part.
// 
// Additional Comments: 
//
////////////////////////////////////////////////////////////////////////////////

module CPLD1004 #(
    parameter pMode = 3'b001,       // Mstr SPI: 001; JTAG: 101; Slave Ser: 111
    parameter pVSel = 3'b101,       // Normal SPI Read (READ = 0x03)
    parameter pPUDC = 1'b1          // Enable PUs during configuration
)(
    input   nPROG,
    input   DONE,
    
    output  [2:0] M,
    output  [2:0] VS,
    output  nPUDC,
    
    output  nWP_SIO2,
    output  nHold_SIO3,
    
    input   [2:0] nCSO,
//    input   SCK,
//    input   MOSI,
//    output  MISO,
    
    output  nCS,
//    input   nRst,
//    input   Phi2O,
    
//    input   nRstO,
    
//    input   [3:1] XA,
//    
//    input   Sync,
//    input   [3:2] CE,
    
    output  Rdy,
    output  nNMI,
    output  nIRQ,
    output  nSO
);

////////////////////////////////////////////////////////////////////////////////
//
//  Declarations
//

wire    Config;

////////////////////////////////////////////////////////////////////////////////
//
//  Implementation
//

assign Config = (~nPROG | ~DONE);

assign M[2]  = ((Config) ? ((pMode[2]) ? 1'bZ : 1'b0) : 1'bZ);
assign M[1]  = ((Config) ? ((pMode[1]) ? 1'bZ : 1'b0) : 1'bZ);
assign M[0]  = ((Config) ? ((pMode[0]) ? 1'bZ : 1'b0) : 1'bZ);

assign VS[2] = ((Config) ? ((pVSel[2]) ? 1'bZ : 1'b0) : 1'bZ);
assign VS[1] = ((Config) ? ((pVSel[1]) ? 1'bZ : 1'b0) : 1'bZ);
assign VS[0] = ((Config) ? ((pVSel[0]) ? 1'bZ : 1'b0) : 1'bZ);

assign nPUDC = ((Config) ? ((pPUDC)    ? 1'b0 : 1'bZ) : 1'bZ);

assign nWP_SIO2   = 1'b1;
assign nHold_SIO3 = 1'b1;

assign nCS  = &nCSO[2:1];

//assign MISO = 1'bZ;
    
assign Rdy  = 1'b1;
assign nNMI = 1'b1;
assign nIRQ = 1'b1;
assign nSO  = 1'b1;

endmodule
